package com.openexchange.custom.parallels.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;
import org.apache.xmlrpc.client.XmlRpcCommonsTransportFactory;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.openexchange.api2.OXException;
import com.openexchange.config.ConfigurationService;
import com.openexchange.custom.parallels.osgi.ParallelsServiceRegistry;
import com.openexchange.groupware.AbstractOXException;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.ldap.LdapException;
import com.openexchange.groupware.ldap.User;
import com.openexchange.groupware.ldap.UserStorage;
import com.openexchange.server.ServiceException;
import com.openexchange.session.Session;
import com.openexchange.tools.servlet.AjaxException;

/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/**
 * 
 * Handles requests to the OpenAPI interface of POA.
 * 
 * Currently the management of Black/White Lists is implemented
 * 
 */
public final class ParallelsOpenApiServletRequest  {
	
	private static final Log LOG = LogFactory.getLog(ParallelsOpenApiServletRequest.class);	
	private User user;	
	private ConfigurationService configservice;	
	
	
	public static final String ACTION_ADD = "add"; // use this to add data to the specified type if list	
	public static final String ACTION_DELETE = "delete"; // use this to delete data to the specified type if list	
	public static final String ACTION_GET = "get"; // use this to get data for the specified type if list
	public static final String PARAMETER_DATA = "data"; // use this to send data
	
	public static final String MODULE_WHITELIST= "whitelist"; // whiteliste module
	public static final String MODULE_BLACKLIST = "blacklist"; // blacklist module	
		
	public static final String PROPERTY_OPENAPI_INTERFACE_URL = "com.openexchange.custom.parallels.openapi.interface_url";
	public static final String PROPERTY_OPENAPI_INTERFACE_AUTH_ID = "com.openexchange.custom.parallels.openapi.auth_id";
	public static final String PROPERTY_OPENAPI_INTERFACE_AUTH_PASSWORD = "com.openexchange.custom.parallels.openapi.auth_password";
	public static final String PROPERTY_OPENAPI_INTERFACE_AUTH_ENABLED = "com.openexchange.custom.parallels.openapi.auth_enabled";
	
	public ParallelsOpenApiServletRequest(final Session sessionObj, final Context ctx)	throws OXException, ServiceException {
		try {
			this.user = UserStorage.getInstance().getUser(sessionObj.getUserId(), ctx);
		} catch (final LdapException e) {
			LOG.error(e.getMessage(), e);
			throw new OXException(e);
		}		
		
		// init config 
		this.configservice = ParallelsServiceRegistry.getServiceRegistry().getService(ConfigurationService.class,true); 
	}
	
	public Object action(final String action,String module, final JSONObject jsonObject) throws AbstractOXException, JSONException {
		Object retval = null;
		
		if (module.equalsIgnoreCase(MODULE_WHITELIST)) {			
			if(action.equalsIgnoreCase(ACTION_ADD)){				
				retval = addOrDeleteFromList("pem.spam_assassin.addItems", "white", jsonObject);
			}else if (action.equalsIgnoreCase(ACTION_DELETE)){				
				retval = addOrDeleteFromList("pem.spam_assassin.deleteItems", "white", jsonObject);
			}else if (action.equalsIgnoreCase(ACTION_GET)){
				retval = getItemsForList("white"); 
			}else{
				throw new AjaxException(AjaxException.Code.UnknownAction, module);
			}			
		}else if(module.equalsIgnoreCase(MODULE_BLACKLIST)){
			if(action.equalsIgnoreCase(ACTION_ADD)){				
				retval = addOrDeleteFromList("pem.spam_assassin.addItems", "black", jsonObject);
			}else if (action.equalsIgnoreCase(ACTION_DELETE)){
				retval = addOrDeleteFromList("pem.spam_assassin.deleteItems", "black", jsonObject);
			}else if (action.equalsIgnoreCase(ACTION_GET)){
				retval = getItemsForList("black"); 
			}else{
				throw new AjaxException(AjaxException.Code.UnknownAction, module);
			}
		}else{
			throw new AjaxException(AjaxException.Code.UNKNOWN_MODULE, module);
		}
		return retval;
	}
	
	
	
	
	private Object addOrDeleteFromList(String method,String list,JSONObject jsonObject) throws ParallelsOpenApiServletException, JSONException, AjaxException {
		
		// we need the data parameter here
		checkForMissingParameter(jsonObject);
		
		try{				
			
			
			Map bla = new HashMap();
			String[] data = {jsonObject.getString("data")};
			bla.put("items",data );
			bla.put("list",list);
			
			
			bla.put("login", this.user.getMail());
			Map response = (HashMap) getRpcClient().execute(method,new Object[]{bla});
			checkXMLRpcResponseForError(response);
		} catch (XmlRpcException e) {
			LOG.error("xml-rpc error detected while communicating with openapi interface");
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.HTTP_COMMUNICATION_ERROR, e.getMessage());     
		} catch (MalformedURLException e) {
			LOG.error("IO error occured while communicating with openapi interface");
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.HTTP_COMMUNICATION_ERROR, e.getMessage());    
		} catch (ServiceException e) {
			LOG.error("internal configuration error occured");
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.OPENAPI_COMMUNICATION_ERROR, e.getMessage());     
		}finally{			
		}
		
		return null;
	}
	
	/**
	 * checks if response contains error field
	 * @param response
	 * @throws ParallelsOpenApiServletException
	 */
	private void checkXMLRpcResponseForError(Map response) throws ParallelsOpenApiServletException{
		if(response.containsKey("error_code")){
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.OPENAPI_COMMUNICATION_ERROR, response.get("error_message")+" (OPEN_API_ERROR_CODE:"+response.get("error_code")+" )");
		}
	}
	
	private Object getItemsForList(String list) throws ParallelsOpenApiServletException, JSONException, AjaxException {
		
		JSONObject json_response = new JSONObject();
		
		try{				
			
			
			
			Map bla = new HashMap();			
			bla.put("list",list);			
			bla.put("login", this.user.getMail());
			Map response = (HashMap) getRpcClient().execute("pem.spam_assassin.getItems",new Object[]{bla});
			checkXMLRpcResponseForError(response); 
			
			JSONArray retval = new JSONArray();
			
			if(response.containsKey("result")){
				HashMap tmp_ = (HashMap)response.get("result");
				Object[] tmp__ = (Object[])tmp_.get("list");
				for (Object addy : tmp__){
					retval.put(addy.toString());
				}
			}
			if(LOG.isDebugEnabled()){
				LOG.debug("Got the following items from openapi for list \""+list+"\" "+retval.toString());
			}
			json_response.put("items", retval);
			return json_response;
		} catch (XmlRpcException e) {
			LOG.error("xml-rpc error detected while communicating with openapi interface");
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.HTTP_COMMUNICATION_ERROR, e.getMessage());     
		} catch (MalformedURLException e) {
			LOG.error("IO error occured while communicating with openapi interface");
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.HTTP_COMMUNICATION_ERROR, e.getMessage());    
		} catch (ServiceException e) {
			LOG.error("internal configuration error occured");
			throw new ParallelsOpenApiServletException(ParallelsOpenApiServletException.Code.OPENAPI_COMMUNICATION_ERROR, e.getMessage());     
		}finally{			
		}
		
		
	}
	
	private XmlRpcClient getRpcClient() throws MalformedURLException, ServiceException{
		XmlRpcClientConfigImpl config = new XmlRpcClientConfigImpl();
		config.setServerURL(new URL(getOpenAPIInterfaceURL()));
		config.setUserAgent("OX-HE POA PLUGIN");
		if(isOpenAPIAuthEnabled()){
			if(LOG.isDebugEnabled()){
				LOG.debug("Using HTTP BASIC AUTH (Username: "+getOpenAPIAuthID()+") for sending XML-RPC requests to OpenAPI...");
			}
			config.setBasicUserName(getOpenAPIAuthID());
			config.setBasicPassword(getOpenAPIAuthPassword());
		}
		XmlRpcClient client = new XmlRpcClient();
		client.setTransportFactory(new XmlRpcCommonsTransportFactory(client));
		client.setConfig(config);
		return client;
	}
	
	private void checkForMissingParameter(JSONObject jsonObject) throws AjaxException{
		if(!jsonObject.has(PARAMETER_DATA)){
			throw new AjaxException(AjaxException.Code.MISSING_PARAMETER, "data");
		}
	}

	
	private String getOpenAPIInterfaceURL() throws ServiceException{
		return getFromConfig(PROPERTY_OPENAPI_INTERFACE_URL);
	}
	
	private String getOpenAPIAuthID() throws ServiceException{
		return getFromConfig(PROPERTY_OPENAPI_INTERFACE_AUTH_ID);
	}	
	
	private String getOpenAPIAuthPassword() throws ServiceException{
		return getFromConfig(PROPERTY_OPENAPI_INTERFACE_AUTH_PASSWORD);
	}
	
	private boolean isOpenAPIAuthEnabled() throws ServiceException{
		return this.configservice.getBoolProperty(PROPERTY_OPENAPI_INTERFACE_AUTH_ENABLED, false);
	}
	

	private String getFromConfig(String key) throws ServiceException{		
		return this.configservice.getProperty(key); 
	}
	
	

	

}
